package hikapro.com.backpack.presenter;

import android.content.Context;
import android.os.Bundle;
import android.support.v7.widget.LinearLayoutManager;
import android.support.v7.widget.RecyclerView;
import android.support.v7.widget.helper.ItemTouchHelper;
import android.view.LayoutInflater;
import android.view.ViewGroup;
import android.widget.Toast;

import java.lang.ref.WeakReference;

import hikapro.com.backpack.R;
import hikapro.com.backpack.model.Model;
import hikapro.com.backpack.model.entities.Item;
import hikapro.com.backpack.model.entities.Set;
import hikapro.com.backpack.presenter.adapters.helper.items.DividerDecoration;
import hikapro.com.backpack.presenter.adapters.ItemListAdapter;
import hikapro.com.backpack.presenter.adapters.helper.items.ItemSwipeCallback;
import hikapro.com.backpack.presenter.adapters.helper.items.StickyHeaderDecoration;
import hikapro.com.backpack.view.View;

/**
 * Created by tariel on 20/04/16.
 */
public class ItemListPresenter implements Presenter.ItemList {

    private static final String BUNDLE_SET_LIST_KEY = "BUNDLE_SET_LIST_KEY";

    private WeakReference<View.ItemList>    view;
    private Model.Item                      model;
    private Set                             set;
    private ItemListAdapter                 adapter;
    RecyclerView                            recycler;

    public ItemListPresenter() {
        this.adapter = new ItemListAdapter(this);
        adapter.setHasStableIds(true);
    }

    // life cycle -->

    @Override
    public void onDestroy(boolean isChangingConfiguration) {
        view = null;
        model.onDestroy(isChangingConfiguration);
        if ( !isChangingConfiguration ) {
            model = null;
        }
    }
    @Override
    public android.view.View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        /*
        if (savedInstanceState != null)
            set = (Set) savedInstanceState.getSerializable(BUNDLE_SET_LIST_KEY);*/
        android.view.View view = inflater.inflate(R.layout.fragment_item_list, container, false);
        LinearLayoutManager llm = new LinearLayoutManager(getActivityContext());
        recycler = (RecyclerView) view.findViewById(R.id.items_recycler);
        recycler.setLayoutManager(llm);
        recycler.setAdapter(adapter);

        final StickyHeaderDecoration decoration = new StickyHeaderDecoration(adapter);
        recycler.addItemDecoration(decoration);
        recycler.addItemDecoration(new DividerDecoration(getActivityContext()));
        adapter.registerAdapterDataObserver(new RecyclerView.AdapterDataObserver() {
            @Override
            public void onChanged() {
                decoration.invalidateHeaders();
            }
        });
        ItemSwipeCallback itemSwipeCallback = new ItemSwipeCallback(0, ItemTouchHelper.LEFT, adapter, getActivityContext());

        ItemTouchHelper itemTouchHelper = new ItemTouchHelper(itemSwipeCallback);
        itemTouchHelper.attachToRecyclerView(recycler);
        adapter.setUndoOn(true);

        model.executeQuery();
        return view;
    }
    @Override
    public void onSaveInstanceState(Bundle outState) {
        if (set != null)
            outState.putSerializable(BUNDLE_SET_LIST_KEY, set);
    }

    // life cycle <--

    // process -->

    @Override
    public void notifyDataSetChanged() {
        adapter.notifyDataSetChanged();
    }

    @Override
    public void clickItem(int itemId) {
        Item item = model.findItem(itemId);
        if (item != null)
            getView().showItemDetail(item);
        else
            showMessage(String.format("Item with Id %d is not found.", itemId));
    }
    // process <--

    // other impl -->

    @Override
    public void setView(View.ItemList view) {
        this.view = new WeakReference<>(view);
        this.set = getView().getSet();
    }
    @Override
    public void setModel(Model.Item model) {
        this.model = model;
    }

    @Override
    public Model.Item getModel() {
        return model;
    }

    @Override
    public Context getAppContext() {
        try {
            return getView().getAppContext();
        } catch (NullPointerException e) {
            return null;
        }
    }
    @Override
    public Context getActivityContext() {
        try {
            return getView().getActivityContext();
        } catch (NullPointerException e) {
            return null;
        }
    }
    @Override
    public void showMessage(String message) {
        Toast.makeText(getView().getAppContext(), message, Toast.LENGTH_SHORT).show();
    }

    // other impl <--

    private View.ItemList getView() throws NullPointerException {
        if ( view != null )
            return view.get();
        else
            throw new NullPointerException("View is unavailable");
    }

    @Override
    public void filter(String query) {
        adapter.filter(query);
        recycler.scrollToPosition(0);
    }

    @Override
    public Set getCurrentSet() {
        return set;
    }
}
